Class: Dalli::Socket::UNIX

Inherits:
UNIXSocket
  • Object
show all
Includes:
InstanceMethods
Defined in:
lib/dalli/socket.rb,
lib/dalli/socket.rb

Overview

UNIX represents a UNIX domain socket, which is an interprocess communication mechanism between processes on the same host. Used when the Memcached server is running on the same machine as the Dalli client.

Constant Summary

Constants included from InstanceMethods

InstanceMethods::FILTERED_OUT_OPTIONS, InstanceMethods::WAIT_RCS

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from InstanceMethods

#append_to_buffer?, #logged_options, #nonblock_timed_out?, #read_available, #readfull

Constructor Details

#initialize(*_args) ⇒ UNIX

Returns a new instance of UNIX.

Raises:



207
208
209
# File 'lib/dalli/socket.rb', line 207

def initialize(*_args)
  raise Dalli::DalliError, 'Unix sockets are not supported on Windows platform.'
end

Instance Attribute Details

#optionsObject

options - supports enhanced logging in the case of a timeout server - used to support IO.select in the pipelined getter



223
224
225
# File 'lib/dalli/socket.rb', line 223

def options
  @options
end

Class Method Details

.init_socket_options(sock, options) ⇒ Object



234
235
236
237
238
# File 'lib/dalli/socket.rb', line 234

def self.init_socket_options(sock, options)
  # https://man7.org/linux/man-pages/man7/unix.7.html
  sock.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_SNDBUF, options[:sndbuf]) if options[:sndbuf]
  sock.timeout = options[:socket_timeout] if options[:socket_timeout] && sock.respond_to?(:timeout=)
end

.open(path, options = {}) ⇒ Object



225
226
227
228
229
230
231
232
# File 'lib/dalli/socket.rb', line 225

def self.open(path, options = {})
  Timeout.timeout(options[:socket_timeout]) do
    sock = new(path)
    sock.options = { path: path }.merge(options)
    init_socket_options(sock, options)
    sock
  end
end